#! urs/bin/perl -w
# Created by Yiming Cheng on 2006/1/10
# Function: predict the positive or negative of the results based on 30/10 from SVM prediction
# Comments: modified on 2006/05/06

use strict;
use Getopt::Std;

sub exit_help{
	print "\nUsage: perl predict_m.pl -i input -c num_file -o output [-e e_cutoff]\n";
	print "Options:\n";
	print "-i input --- the input file generted directly from svm-predict.\n";
	print "-c num_file --- the file geneated by match_seq.pl including the hit number of the sequences\n";
	print "-o output --- the simpliest output file for mode 's' in polya_svm. For other mode,we change the output in polya_svm.\n";
	print "-e cutoff --- cut off e_value[0---1]. Default is 0.5\n\n";
	exit;
}

my %args;
getopt("icoe",\%args);

my ($input,$output,$num_file,$e_cutoff);
&ini_options();

my (@pro_data,$pos_col);
&read_input();

my $num_line = 0;
my $pos=0;
&read_num_file(); #generate the temptory output file

open(OUT,">$output");
&print_output();

sub ini_options{
	$input = $args{i};
	if (!defined($input)){
		&exit_help();
	}
	$num_file = $args{c};
	if (!defined($num_file)){
		&exit_help();
	}
	$output = $args{o};
	if (!defined($output)){
		&exit_help();
	}

	$e_cutoff = $args{e};
	if (!defined($e_cutoff)){
		$e_cutoff = 0.5;
	}
}

sub read_input{
	open(IN,"<$input");
	while(<IN>){
		chomp;
		my @items = split(/ /,$_);
		if ($. == 1){
			if ($items[1] == 1){
				$pos_col = 1;
			} elsif ($items[1] == -1){
				$pos_col = 2;
			} else {
				die "Data has some problem.\n";
			}
			next;
		}
		push @pro_data,$items[$pos_col];
	}
	close(IN);
}

sub read_num_file{
	open(NUM,"<$num_file")||die "Can't open the number file.\n";
	my $cur_num = 0;
	my @temp_return;
	my $output_junk = $output.".cym";
	open(OUT_JUNK,">$output_junk");
	while(<NUM>){
		chomp;
		$num_line++;
		my @num_tmp = split(/\t/,$_);  # id, num_hit,[location]
		print OUT_JUNK ">",$num_tmp[0],"\n";
		if ($num_tmp[1] ==0){
			print OUT_JUNK "Seq too short.\n";
			next;
		} elsif ($num_tmp[1] ==1){
			if ($pro_data[$cur_num]>0.5){
				if ($num_tmp[2] != 0){
					print OUT_JUNK ($num_tmp[2]-100),":",($num_tmp[2]-100),":",($num_tmp[2]-100),":",$pro_data[$cur_num],"\n";
				} else {
					print OUT_JUNK "M:M:M:",$pro_data[$cur_num],"\n";
				}
				$pos++;
			} else {
				print OUT_JUNK "N\n";
			}
		} else {
			@temp_return = &is_pos_m($cur_num,$num_tmp[1],$e_cutoff);
			if ($temp_return[$#temp_return]>0){
				$pos++;
				print OUT_JUNK join("\t",@temp_return[0 .. ($#temp_return-1)]),"\n";
			} else {
				print OUT_JUNK "N\n";
			}
		}
		$cur_num = $cur_num + $num_tmp[1];
	}
	close(NUM);
	close(OUT_JUNK);
}

sub print_output{
	print OUT "#Total number of sequences: $num_line.\n";
	print OUT "#Positive sequences: $pos.\n";
	print OUT "#The e-value cut-off is $e_cutoff.\n\n";
	
	my $output_junk = $output.".cym";
	open(OUT_JUNK,"<$output_junk")||die "Can not open file.cym\n";
	while(<OUT_JUNK>){
		chomp;
		if ($_ =~ /^>/){
			print OUT $_,"\n";
			print OUT "#+/-\tFrom \t To \t Max-Location\t E-value\n";
		} elsif ($_ =~ /^S/){
			print OUT "Seq too short, need to be greater than 120 nts","\n\n";
		} elsif ($_ =~ /^N/){   # Negative
			print OUT "-\n\n";
		} elsif ($_ =~ /^M/){
			my @line_junk = split(/:/,$_);
			print OUT "+\tM\tM\tM\t",sprintf("%.3f",(1-$line_junk[3])),"\n";
		} elsif ($_ =~ /^\d+/){
			my @line_junk = split(/\t/,$_);
			for (my $i=0;$i<($#line_junk+1);$i++){
				if ($line_junk[$i] =~ /^(\d+):(\d+):(\d+):([\d\.]+)/){
					print OUT "+ \t",($1+100),"\t",($2+100),"\t",($3+100),"\t\t",sprintf("%.3f",(1-$4)),"\n";
				}
			}
			print OUT "\n";
		} else {
			die "Never reach in print_output.\n";
		}
	}	
	close(OUT_JUNK);
	system("rm $output_junk -f");
	close(OUT);
}

sub is_pos_m{
	my $start = $_[0];
	my $len = $_[1];
	my $cut_off = $_[2];
	my $pos_win_size;
	my $time_up_size;
	if ($len > 50){
		$pos_win_size = 30;
		$time_up_size = 10;
	} elsif ($len > 10){
		$pos_win_size = 3*int($len*0.1);
		$time_up_size = int($pos_win_size/3);
	} elsif ($len>4) {
		$pos_win_size = 2;
		$time_up_size = 1;
	} else {
		$pos_win_size = 1;
		$time_up_size = 1;
	}
	my @vec = @pro_data[$start .. ($start+$len-1)];
	my $vec_len = @vec;
	my $product=1;
	my $max_product=$cut_off;
	my ($k,$j,$m);
	my @return_arr;
	my $return_pos=0;
	my $pos_size=0;
	my $region_win=0;
	my $region_win_start =0;
	for ($k=0;$k<$vec_len;$k++){
		if ($vec[$k]>=0.5){
			if ($region_win == 0){
				$region_win_start = $k;
			}
			$region_win++;
		}
		if (($k==($vec_len-1))||($vec[$k]<0.5)){
			if ($region_win>=$pos_win_size){
				$product = 1;
				my @region_pro=@vec[$region_win_start .. $k];
				my $max_j =0;
				for ($j=0;$j<($region_win-$time_up_size+1);$j++){
					for ($m=0;$m<$time_up_size;$m++){
						$product = $product*$region_pro[$j+$m];
					}
					if ($product>$max_product){
						$max_product = $product;
						$max_j = $j+int($time_up_size/2);
					}
					$product=1;	
				}
				if ($max_product >$cut_off){
					my $temp_ret = $region_win_start.":".($region_win_start+$region_win).":".($region_win_start+$max_j).":".sprintf("%.3f",$max_product);
					push(@return_arr,$temp_ret);
					$return_pos++;
				}
				$max_product = $cut_off;
				$region_win = 0;
			} else {
				$region_win=0;
			}
		}

	}
	push(@return_arr,$return_pos);
	return(@return_arr);
}

