#! /usr/bin/perl -w 
# Created by Yiming Cheng on 2006/02/17
# Function: scale the test data and train data
# Comments: modified on 2006/05/09. Generate own mean and sd function.

use strict;
use Getopt::Std;

my %args;
getopt("iosr",\%args);

sub exit_help{
	print "\nUsage: perl scale.pl -i input -o output {[-s save_para] | [-r restor_para]}\n";
	print "Options:\n";
	print "-i input --- input file to scale.\n";
	print "-o ouput --- output file that has been scaled.\n";
	print "-s save_para --- scale the input file and save the parameters to the file save_para.\n";
	print "-r restore_para --- use the restore_para to scale the input file.\n\n";
	exit;
}
my ($input,$output,$restore,$save);
&ini_options();

sub ini_options{
	$input = $args{i};
	if (!defined($input)){
		&exit_help();
	}
	$output = $args{o};
	if (!defined($output)){
		&exit_help();
	}
	$save = $args{s};
	$restore = $args{r};
	if ((!defined($save))&&(!defined($restore))){
		&exit_help();
	}
}

my (%data,%sta_data);

if (defined($save)){
	open(IN,"<$input");
	open(OUT,">$output");
	open(SAVE,">$save");
	my $num_items;
	while(<IN>){
		chomp;
		my @items = split(/\t/,$_);
		$num_items = @items;
		for (my $i=1;$i<$num_items;$i++){
			push @{$data{$i}},$items[$i];
		}
	}
	close(IN);

	for (my $i=1;$i<$num_items;$i++){
		my @one_data =@{$data{$i}};
		my ($mean,$sd) = &mean_sd(join(":",@one_data));
		$sta_data{$i}{"mean"}=$mean;
		$sta_data{$i}{"sd"}=$sd;
		print SAVE "$i\t$mean\t$sd\n";		
	}
	close(SAVE);
	open(IN,"<$input");
	while(<IN>){
		chomp;
		my @items = split(/\t/,$_);
		print OUT "$items[0] ";
		for (my $i=1;$i<$num_items;$i++){
			my $item_i = ($items[$i]-$sta_data{$i}{"mean"})/$sta_data{$i}{"sd"};
			print OUT "$i:",sprintf("%.3f",$item_i)," ";
		}
		print OUT "\n";
	}
	close(IN);
	close(OUT);

} elsif (defined($restore)){
	open(RESTORE,"<$restore");
	while(<RESTORE>){
		chomp;
		my @items = split(/\t/,$_);	
		$sta_data{$items[0]}{"mean"}=$items[1];
		$sta_data{$items[0]}{"sd"}=$items[2];
	}
	close(RESTORE);

	open(IN,"<$input");
	open(OUT,">$output");
	while(<IN>){
		chomp;
		my @items = split(/\t/,$_);
		my $num_items = @items;
		print OUT "$items[0] ";
		for (my $i=1;$i<$num_items;$i++){
			my $item_i = ($items[$i]-$sta_data{$i}{"mean"})/$sta_data{$i}{"sd"};
			print OUT "$i:",sprintf("%.3f",$item_i)," ";
		}
		print OUT "\n";
	}
	close(IN);
	close(OUT);
} else {
	print "You should give the save file or retore file name";
}

sub mean_sd{
	my @data = split(/:/,$_[0]);
	my $num_data = @data;
	my ($sum_data,$data_2);
	my ($t_mean,$t_sd);
	for (my $i=0;$i<$num_data;$i++){
		$sum_data += $data[$i];
		$data_2 += $data[$i]**2;
	}
	$t_mean = $sum_data/$num_data;
	$t_sd = sqrt(($data_2-$num_data*($t_mean**2))/($num_data-1));
	return($t_mean,$t_sd);
}	
